@@ -94,15 +94,6 @@ pub enum Error {
 
     /// No space for the region
     OutOfSpace,
-
-    /// Not inside the address space
-    Overflow,
-
-    /// Overlap with the existing regions
-    Overlap,
-
-    /// Invalid region
-    InvalidRegion,
 }
 
 /// A virtual memory map ledger.
@@ -143,7 +134,8 @@ impl<const N: usize> Ledger<N> {
         Ok(())
     }
 
-    /// Create a new instance.
+    /// Create a new instance. Make sure that `region.start < region.end`,
+    /// because otherwise the ledger will exhibit undefined behaviour.
     pub const fn new(region: Region) -> Self {
         Self {
             records: [Record::EMPTY; N],
@@ -152,6 +144,49 @@ impl<const N: usize> Ledger<N> {
         }
     }
 
+    /// Check whether the ledger contains the given region, and return the
+    /// maximum allowed access for it. Any empty space will result `None`.
+    pub fn contains(
+        &self,
+        addr: Address<usize, Page>,
+        length: Offset<usize, Page>,
+    ) -> Option<Access> {
+        let region = Region::new(addr, addr + length);
+        let mut access = Access::all();
+        let mut start = region.start;
+
+        if !self.region.contains(&region) {
+            return None;
+        }
+
+        for record in self.records() {
+            if let Some(slice) = record.region.intersection(Region::new(start, region.end)) {
+                if start != slice.start {
+                    return None;
+                }
+
+                start = slice.end;
+                access &= record.access;
+
+                if start == region.end {
+                    return Some(access);
+                }
+            }
+        }
+
+        None
+    }
+
+    /// Check whether the existing reserved addresses in the ledger overlap with the
+    /// given region.
+    pub fn overlaps(&self, addr: Address<usize, Page>, length: Offset<usize, Page>) -> bool {
+        let region = Region::new(addr, addr + length);
+
+        self.records()
+            .iter()
+            .any(|record| region.start < record.region.end && region.end > record.region.start)
+    }
+
     /// Get an immutable view of the records.
     pub fn records(&self) -> &[Record] {
         &self.records[..self.tail]
@@ -180,20 +215,20 @@ impl<const N: usize> Ledger<N> {
         Ok(())
     }
 
-    /// Adds a new record to the ledger, potentially merging with existing records.
-    pub fn map(&mut self, region: Region, access: Access) -> Result<(), Error> {
-        // Make sure the record is valid.
-        if region.start >= region.end {
-            return Err(Error::InvalidRegion);
-        }
+    /// Reserve an address range from the ledger. When overlapping with an
+    /// existing record, the new access will be over-written. Conserves space by
+    /// merging the adjacent records in the ledger after the reservation has
+    /// been done.
+    pub fn map(
+        &mut self,
+        addr: Address<usize, Page>,
+        length: Offset<usize, Page>,
+        access: Access,
+    ) -> Result<(), Error> {
+        let region = Region::new(addr, addr + length);
 
-        // Make sure the record fits in our adress space.
-        if !self.region.contains(&region) {
-            return Err(Error::Overflow);
-        }
-
-        // Clear out the space for the new record.
-        if let Err(err) = self.unmap(region) {
+        // Clear out the possibly reserved space for the new record.
+        if let Err(err) = self.unmap(addr, length) {
             return Err(err);
         }
 
@@ -302,14 +337,12 @@ impl<const N: usize> Ledger<N> {
     }
 
     /// Delete sub-regions.
-    pub fn unmap(&mut self, region: Region) -> Result<(), Error> {
-        if region.start >= region.end {
-            return Err(Error::InvalidRegion);
-        }
-
-        if region.start < self.region.start || region.end > self.region.end {
-            return Err(Error::Overflow);
-        }
+    pub fn unmap(
+        &mut self,
+        addr: Address<usize, Page>,
+        length: Offset<usize, Page>,
+    ) -> Result<(), Error> {
+        let region = Region::new(addr, addr + length);
 
         let mut index = 0;
 
@@ -449,6 +482,77 @@ mod tests {
         assert!(equal);
     }
 
+    #[rstest::rstest]
+    #[case(&[(0x3, 0x6, N), (0x6, 0xa, R), (0xa, 0xd, N)], (0x0, 0x0d, None))]
+    #[case(&[(0x3, 0x6, N), (0x6, 0xa, R), (0xa, 0xd, N)], (0x3, 0x10, None))]
+    #[case(&[(0x3, 0x6, N), (0x6, 0xa, R), (0xa, 0xd, N)], (0x0, 0x10, None))]
+    #[case(&[(0x3, 0x6, N), (0x6, 0xa, R), (0xa, 0xd, N)], (0x3, 0xd, Some(N)))]
+    #[case(&[(0x3, 0x6, N), (0x6, 0xa, R), (0xa, 0xd, N)], (0x6, 0xa, Some(R)))]
+    fn contains(
+        #[case] maps: &[(usize, usize, Access)],
+        #[case] expected: (usize, usize, Option<Access>),
+    ) {
+        let mut ledger = EMPTY_LEDGER.clone();
+        for record in maps
+            .iter()
+            .cloned()
+            .map(|r| Record {
+                region: Region::new(Address::new(r.0 << 12), Address::new(r.1 << 12)),
+                access: r.2,
+            })
+            .collect::<Vec<_>>()
+        {
+            let addr = record.region.start;
+            let length = record.region.end - record.region.start;
+            ledger.map(addr, length, record.access).unwrap();
+        }
+
+        println!("Maps:");
+        trace_records(&ledger.records);
+        println!("Region:");
+        println!("({:>#08x}, {:>#08x})", expected.0 << 12, expected.1 << 12,);
+
+        let access = ledger.contains(
+            Address::new(expected.0 << 12),
+            Offset::from_items(expected.1 - expected.0),
+        );
+        assert_eq!(access, expected.2);
+    }
+
+    #[rstest::rstest]
+    #[case(&[(0x3, 0x6, N), (0x6, 0xa, R), (0xa, 0xd, N)], (0x0, 0x10, true))]
+    #[case(&[(0x3, 0x6, N), (0x6, 0xa, R), (0xa, 0xd, N)], (0x2, 0x7, true))]
+    #[case(&[(0x3, 0x6, N), (0x6, 0xa, R), (0xa, 0xd, N)], (0xc, 0xe, true))]
+    #[case(&[(0x3, 0x6, N), (0x6, 0xa, R), (0xa, 0xd, N)], (0xd, 0xe, false))]
+    #[case(&[(0x3, 0x6, N), (0x6, 0xa, R), (0xa, 0xd, N)], (0x2, 0x3, false))]
+    fn overlaps(#[case] maps: &[(usize, usize, Access)], #[case] expected: (usize, usize, bool)) {
+        let mut ledger = EMPTY_LEDGER.clone();
+        for record in maps
+            .iter()
+            .cloned()
+            .map(|r| Record {
+                region: Region::new(Address::new(r.0 << 12), Address::new(r.1 << 12)),
+                access: r.2,
+            })
+            .collect::<Vec<_>>()
+        {
+            let addr = record.region.start;
+            let length = record.region.end - record.region.start;
+            ledger.map(addr, length, record.access).unwrap();
+        }
+
+        println!("Maps:");
+        trace_records(&ledger.records);
+        println!("Region:");
+        println!("({:>#08x}, {:>#08x})", expected.0 << 12, expected.1 << 12,);
+
+        let access = ledger.overlaps(
+            Address::new(expected.0 << 12),
+            Offset::from_items(expected.1 - expected.0),
+        );
+        assert_eq!(access, expected.2);
+    }
+
     #[rstest::rstest]
     #[case(&[(0x3, 0x6, N), (0xa, 0xd, N)], &[(0x3, 0x6, N), (0xa, 0xd, N)])] // normal insert
     #[case(&[(0x3, 0x6, N), (0xa, 0xd, N), (0x0, 0x1, N)], &[(0x0, 0x1, N), (0x3, 0x6, N), (0xa, 0xd, N)])] // normal insert
@@ -505,7 +609,9 @@ mod tests {
         trace_records(&maps);
 
         for record in maps {
-            ledger.map(record.region, record.access).unwrap();
+            let addr = record.region.start;
+            let length = record.region.end - record.region.start;
+            ledger.map(addr, length, record.access).unwrap();
         }
 
         println!("Result:");
@@ -551,7 +657,9 @@ mod tests {
         trace_regions(&unmaps);
 
         for region in unmaps {
-            ledger.unmap(region).unwrap();
+            let addr = region.start;
+            let length = region.end - region.start;
+            ledger.unmap(addr, length).unwrap();
         }
 
         println!("Result:");
@@ -594,13 +702,13 @@ mod tests {
         trace_records(&maps);
 
         for record in maps {
-            ledger.map(record.region, record.access).unwrap();
+            let addr = record.region.start;
+            let length = record.region.end - record.region.start;
+            ledger.map(addr, length, record.access).unwrap();
         }
 
         if let Some(addr) = ledger.find_free_front(length) {
-            let end = addr + length;
-            let region = Region::new(addr, end);
-            ledger.map(region, Access::empty()).unwrap();
+            ledger.map(addr, length, Access::empty()).unwrap();
         }
 
         println!("Result:");
@@ -643,13 +751,13 @@ mod tests {
         trace_records(&maps);
 
         for record in maps {
-            ledger.map(record.region, record.access).unwrap();
+            let addr = record.region.start;
+            let length = record.region.end - record.region.start;
+            ledger.map(addr, length, record.access).unwrap();
         }
 
         if let Some(addr) = ledger.find_free_back(length) {
-            let end = addr + length;
-            let region = Region::new(addr, end);
-            ledger.map(region, Access::empty()).unwrap();
+            ledger.map(addr, length, Access::empty()).unwrap();
         }
 
         println!("Result:");
